import CoreText
import Quartz
from PyObjCTools.TestSupport import TestCase, min_os_level, fourcc
import objc


class TestCTFont(TestCase):
    def testTypes(self):
        self.assertIsInstance(CoreText.CTFontRef, objc.objc_class)

    @min_os_level("10.6")
    def testConstants10_6(self):
        self.assertEqual(CoreText.kCTFontOptionsDefault, 0)
        self.assertEqual(CoreText.kCTFontOptionsPreventAutoActivation, 1 << 0)
        self.assertEqual(CoreText.kCTFontOptionsPreferSystemFont, 1 << 2)

    @min_os_level("10.8")
    def testConstants10_8(self):
        self.assertIsInstance(CoreText.kCTBaselineClassHanging, str)
        self.assertIsInstance(CoreText.kCTBaselineClassIdeographicCentered, str)
        self.assertIsInstance(CoreText.kCTBaselineClassIdeographicHigh, str)
        self.assertIsInstance(CoreText.kCTBaselineClassIdeographicLow, str)
        self.assertIsInstance(CoreText.kCTBaselineClassMath, str)
        self.assertIsInstance(CoreText.kCTBaselineClassRoman, str)
        self.assertIsInstance(CoreText.kCTBaselineOriginalFont, str)
        self.assertIsInstance(CoreText.kCTBaselineReferenceFont, str)

        self.assertEqual(CoreText.kCTFontTableAnkr, fourcc(b"ankr"))

        self.assertEqual(CoreText.kCTFontUIFontNone, 0xFFFFFFFF)
        self.assertEqual(CoreText.kCTFontUIFontUser, 0)
        self.assertEqual(CoreText.kCTFontUIFontUserFixedPitch, 1)
        self.assertEqual(CoreText.kCTFontUIFontSystem, 2)
        self.assertEqual(CoreText.kCTFontUIFontEmphasizedSystem, 3)
        self.assertEqual(CoreText.kCTFontUIFontSmallSystem, 4)
        self.assertEqual(CoreText.kCTFontUIFontSmallEmphasizedSystem, 5)
        self.assertEqual(CoreText.kCTFontUIFontMiniSystem, 6)
        self.assertEqual(CoreText.kCTFontUIFontMiniEmphasizedSystem, 7)
        self.assertEqual(CoreText.kCTFontUIFontViews, 8)
        self.assertEqual(CoreText.kCTFontUIFontApplication, 9)
        self.assertEqual(CoreText.kCTFontUIFontLabel, 10)
        self.assertEqual(CoreText.kCTFontUIFontMenuTitle, 11)
        self.assertEqual(CoreText.kCTFontUIFontMenuItem, 12)
        self.assertEqual(CoreText.kCTFontUIFontMenuItemMark, 13)
        self.assertEqual(CoreText.kCTFontUIFontMenuItemCmdKey, 14)
        self.assertEqual(CoreText.kCTFontUIFontWindowTitle, 15)
        self.assertEqual(CoreText.kCTFontUIFontPushButton, 16)
        self.assertEqual(CoreText.kCTFontUIFontUtilityWindowTitle, 17)
        self.assertEqual(CoreText.kCTFontUIFontAlertHeader, 18)
        self.assertEqual(CoreText.kCTFontUIFontSystemDetail, 19)
        self.assertEqual(CoreText.kCTFontUIFontEmphasizedSystemDetail, 20)
        self.assertEqual(CoreText.kCTFontUIFontToolbar, 21)
        self.assertEqual(CoreText.kCTFontUIFontSmallToolbar, 22)
        self.assertEqual(CoreText.kCTFontUIFontMessage, 23)
        self.assertEqual(CoreText.kCTFontUIFontPalette, 24)
        self.assertEqual(CoreText.kCTFontUIFontToolTip, 25)
        self.assertEqual(CoreText.kCTFontUIFontControlContent, 26)

    def testConstants(self):
        self.assertIsInstance(CoreText.kCTFontCopyrightNameKey, str)
        self.assertIsInstance(CoreText.kCTFontFamilyNameKey, str)
        self.assertIsInstance(CoreText.kCTFontSubFamilyNameKey, str)
        self.assertIsInstance(CoreText.kCTFontStyleNameKey, str)
        self.assertIsInstance(CoreText.kCTFontUniqueNameKey, str)
        self.assertIsInstance(CoreText.kCTFontFullNameKey, str)
        self.assertIsInstance(CoreText.kCTFontVersionNameKey, str)
        self.assertIsInstance(CoreText.kCTFontPostScriptNameKey, str)
        self.assertIsInstance(CoreText.kCTFontTrademarkNameKey, str)
        self.assertIsInstance(CoreText.kCTFontManufacturerNameKey, str)
        self.assertIsInstance(CoreText.kCTFontDesignerNameKey, str)
        self.assertIsInstance(CoreText.kCTFontDescriptionNameKey, str)
        self.assertIsInstance(CoreText.kCTFontVendorURLNameKey, str)
        self.assertIsInstance(CoreText.kCTFontDesignerURLNameKey, str)
        self.assertIsInstance(CoreText.kCTFontLicenseNameKey, str)
        self.assertIsInstance(CoreText.kCTFontLicenseURLNameKey, str)
        self.assertIsInstance(CoreText.kCTFontSampleTextNameKey, str)
        self.assertIsInstance(CoreText.kCTFontPostScriptCIDNameKey, str)

        self.assertEqual(CoreText.kCTFontNoFontType, 4_294_967_295)  # (uint32_t)-1
        self.assertEqual(CoreText.kCTFontUserFontType, 0)
        self.assertEqual(CoreText.kCTFontUserFixedPitchFontType, 1)
        self.assertEqual(CoreText.kCTFontSystemFontType, 2)
        self.assertEqual(CoreText.kCTFontEmphasizedSystemFontType, 3)
        self.assertEqual(CoreText.kCTFontSmallSystemFontType, 4)
        self.assertEqual(CoreText.kCTFontSmallEmphasizedSystemFontType, 5)
        self.assertEqual(CoreText.kCTFontMiniSystemFontType, 6)
        self.assertEqual(CoreText.kCTFontMiniEmphasizedSystemFontType, 7)
        self.assertEqual(CoreText.kCTFontViewsFontType, 8)
        self.assertEqual(CoreText.kCTFontApplicationFontType, 9)
        self.assertEqual(CoreText.kCTFontLabelFontType, 10)
        self.assertEqual(CoreText.kCTFontMenuTitleFontType, 11)
        self.assertEqual(CoreText.kCTFontMenuItemFontType, 12)
        self.assertEqual(CoreText.kCTFontMenuItemMarkFontType, 13)
        self.assertEqual(CoreText.kCTFontMenuItemCmdKeyFontType, 14)
        self.assertEqual(CoreText.kCTFontWindowTitleFontType, 15)
        self.assertEqual(CoreText.kCTFontPushButtonFontType, 16)
        self.assertEqual(CoreText.kCTFontUtilityWindowTitleFontType, 17)
        self.assertEqual(CoreText.kCTFontAlertHeaderFontType, 18)
        self.assertEqual(CoreText.kCTFontSystemDetailFontType, 19)
        self.assertEqual(CoreText.kCTFontEmphasizedSystemDetailFontType, 20)
        self.assertEqual(CoreText.kCTFontToolbarFontType, 21)
        self.assertEqual(CoreText.kCTFontSmallToolbarFontType, 22)
        self.assertEqual(CoreText.kCTFontMessageFontType, 23)
        self.assertEqual(CoreText.kCTFontPaletteFontType, 24)
        self.assertEqual(CoreText.kCTFontToolTipFontType, 25)
        self.assertEqual(CoreText.kCTFontControlContentFontType, 26)

        self.assertIsInstance(CoreText.kCTFontVariationAxisIdentifierKey, str)
        self.assertIsInstance(CoreText.kCTFontVariationAxisMinimumValueKey, str)
        self.assertIsInstance(CoreText.kCTFontVariationAxisMaximumValueKey, str)
        self.assertIsInstance(CoreText.kCTFontVariationAxisDefaultValueKey, str)
        self.assertIsInstance(CoreText.kCTFontVariationAxisNameKey, str)

        self.assertIsInstance(CoreText.kCTFontFeatureTypeIdentifierKey, str)
        self.assertIsInstance(CoreText.kCTFontFeatureTypeNameKey, str)
        self.assertIsInstance(CoreText.kCTFontFeatureTypeExclusiveKey, str)
        self.assertIsInstance(CoreText.kCTFontFeatureTypeSelectorsKey, str)
        self.assertIsInstance(CoreText.kCTFontFeatureSelectorIdentifierKey, str)
        self.assertIsInstance(CoreText.kCTFontFeatureSelectorNameKey, str)
        self.assertIsInstance(CoreText.kCTFontFeatureSelectorDefaultKey, str)
        self.assertIsInstance(CoreText.kCTFontFeatureSelectorSettingKey, str)

        self.assertEqual(CoreText.kCTFontTableBASE, fourcc(b"BASE"))
        self.assertEqual(CoreText.kCTFontTableCBDT, fourcc(b"CBDT"))
        self.assertEqual(CoreText.kCTFontTableCBLC, fourcc(b"CBLC"))
        self.assertEqual(CoreText.kCTFontTableCFF, fourcc(b"CFF "))
        self.assertEqual(CoreText.kCTFontTableCFF2, fourcc(b"CFF2"))
        self.assertEqual(CoreText.kCTFontTableCOLR, fourcc(b"COLR"))
        self.assertEqual(CoreText.kCTFontTableCPAL, fourcc(b"CPAL"))
        self.assertEqual(CoreText.kCTFontTableCFF, fourcc(b"CFF "))
        self.assertEqual(CoreText.kCTFontTableDSIG, fourcc(b"DSIG"))
        self.assertEqual(CoreText.kCTFontTableEBDT, fourcc(b"EBDT"))
        self.assertEqual(CoreText.kCTFontTableEBLC, fourcc(b"EBLC"))
        self.assertEqual(CoreText.kCTFontTableEBSC, fourcc(b"EBSC"))
        self.assertEqual(CoreText.kCTFontTableGDEF, fourcc(b"GDEF"))
        self.assertEqual(CoreText.kCTFontTableGPOS, fourcc(b"GPOS"))
        self.assertEqual(CoreText.kCTFontTableGSUB, fourcc(b"GSUB"))
        self.assertEqual(CoreText.kCTFontTableHVAR, fourcc(b"HVAR"))
        self.assertEqual(CoreText.kCTFontTableJSTF, fourcc(b"JSTF"))
        self.assertEqual(CoreText.kCTFontTableLTSH, fourcc(b"LTSH"))
        self.assertEqual(CoreText.kCTFontTableMATH, fourcc(b"MATH"))
        self.assertEqual(CoreText.kCTFontTableMVAR, fourcc(b"MVAR"))
        self.assertEqual(CoreText.kCTFontTableOS2, fourcc(b"OS/2"))
        self.assertEqual(CoreText.kCTFontTablePCLT, fourcc(b"PCLT"))
        self.assertEqual(CoreText.kCTFontTableSTAT, fourcc(b"STAT"))
        self.assertEqual(CoreText.kCTFontTableSVG, fourcc(b"SVG "))
        self.assertEqual(CoreText.kCTFontTableVDMX, fourcc(b"VDMX"))
        self.assertEqual(CoreText.kCTFontTableVORG, fourcc(b"VORG"))
        self.assertEqual(CoreText.kCTFontTableVVAR, fourcc(b"VVAR"))
        self.assertEqual(CoreText.kCTFontTableZapf, fourcc(b"Zapf"))
        self.assertEqual(CoreText.kCTFontTableAcnt, fourcc(b"acnt"))
        self.assertEqual(CoreText.kCTFontTableAnkr, fourcc(b"ankr"))
        self.assertEqual(CoreText.kCTFontTableAvar, fourcc(b"avar"))
        self.assertEqual(CoreText.kCTFontTableBdat, fourcc(b"bdat"))
        self.assertEqual(CoreText.kCTFontTableBhed, fourcc(b"bhed"))
        self.assertEqual(CoreText.kCTFontTableBloc, fourcc(b"bloc"))
        self.assertEqual(CoreText.kCTFontTableBsln, fourcc(b"bsln"))
        self.assertEqual(CoreText.kCTFontTableCidg, fourcc(b"cidg"))
        self.assertEqual(CoreText.kCTFontTableCmap, fourcc(b"cmap"))
        self.assertEqual(CoreText.kCTFontTableCvar, fourcc(b"cvar"))
        self.assertEqual(CoreText.kCTFontTableCvt, fourcc(b"cvt "))
        self.assertEqual(CoreText.kCTFontTableFdsc, fourcc(b"fdsc"))
        self.assertEqual(CoreText.kCTFontTableFeat, fourcc(b"feat"))
        self.assertEqual(CoreText.kCTFontTableFmtx, fourcc(b"fmtx"))
        self.assertEqual(CoreText.kCTFontTableFond, fourcc(b"fond"))
        self.assertEqual(CoreText.kCTFontTableFpgm, fourcc(b"fpgm"))
        self.assertEqual(CoreText.kCTFontTableFvar, fourcc(b"fvar"))
        self.assertEqual(CoreText.kCTFontTableGasp, fourcc(b"gasp"))
        self.assertEqual(CoreText.kCTFontTableGlyf, fourcc(b"glyf"))
        self.assertEqual(CoreText.kCTFontTableGvar, fourcc(b"gvar"))
        self.assertEqual(CoreText.kCTFontTableHdmx, fourcc(b"hdmx"))
        self.assertEqual(CoreText.kCTFontTableHead, fourcc(b"head"))
        self.assertEqual(CoreText.kCTFontTableHhea, fourcc(b"hhea"))
        self.assertEqual(CoreText.kCTFontTableHmtx, fourcc(b"hmtx"))
        self.assertEqual(CoreText.kCTFontTableHsty, fourcc(b"hsty"))
        self.assertEqual(CoreText.kCTFontTableJust, fourcc(b"just"))
        self.assertEqual(CoreText.kCTFontTableKern, fourcc(b"kern"))
        self.assertEqual(CoreText.kCTFontTableKerx, fourcc(b"kerx"))
        self.assertEqual(CoreText.kCTFontTableLcar, fourcc(b"lcar"))
        self.assertEqual(CoreText.kCTFontTableLtag, fourcc(b"ltag"))
        self.assertEqual(CoreText.kCTFontTableLoca, fourcc(b"loca"))
        self.assertEqual(CoreText.kCTFontTableMaxp, fourcc(b"maxp"))
        self.assertEqual(CoreText.kCTFontTableMeta, fourcc(b"meta"))
        self.assertEqual(CoreText.kCTFontTableMort, fourcc(b"mort"))
        self.assertEqual(CoreText.kCTFontTableMorx, fourcc(b"morx"))
        self.assertEqual(CoreText.kCTFontTableName, fourcc(b"name"))
        self.assertEqual(CoreText.kCTFontTableOpbd, fourcc(b"opbd"))
        self.assertEqual(CoreText.kCTFontTablePost, fourcc(b"post"))
        self.assertEqual(CoreText.kCTFontTablePrep, fourcc(b"prep"))
        self.assertEqual(CoreText.kCTFontTableProp, fourcc(b"prop"))
        self.assertEqual(CoreText.kCTFontTableSbit, fourcc(b"sbit"))
        self.assertEqual(CoreText.kCTFontTableTrak, fourcc(b"trak"))
        self.assertEqual(CoreText.kCTFontTableVhea, fourcc(b"vhea"))
        self.assertEqual(CoreText.kCTFontTableVmtx, fourcc(b"vmtx"))
        self.assertEqual(CoreText.kCTFontTableXref, fourcc(b"xref"))

        self.assertEqual(CoreText.kCTFontTableOptionNoOptions, 0)
        self.assertEqual(CoreText.kCTFontTableOptionExcludeSynthetic, (1 << 0))

    def testFunctions(self):
        font = CoreText.CTFontCreateWithName("Optima Bold", 14, None)
        self.assertIsInstance(font, CoreText.CTFontRef)
        self.assertResultIsCFRetained(CoreText.CTFontCreateWithName)

        descriptor = CoreText.CTFontDescriptorCreateWithNameAndSize("Optima Bold", 14.0)
        self.assertIsInstance(descriptor, CoreText.CTFontDescriptorRef)

        font = CoreText.CTFontCreateWithFontDescriptor(descriptor, 5.0, None)
        self.assertIsInstance(font, CoreText.CTFontRef)
        self.assertResultIsCFRetained(CoreText.CTFontCreateWithFontDescriptor)

        font = CoreText.CTFontCreateUIFontForLanguage(
            CoreText.kCTFontMiniSystemFontType, 10.0, "nl_NL"
        )
        self.assertIsInstance(font, CoreText.CTFontRef)
        self.assertResultIsCFRetained(CoreText.CTFontCreateUIFontForLanguage)

        font2 = CoreText.CTFontCreateCopyWithAttributes(font, 9.0, None, None)

        self.assertIsInstance(font2, CoreText.CTFontRef)
        self.assertResultIsCFRetained(CoreText.CTFontCreateCopyWithAttributes)

        font2 = CoreText.CTFontCreateCopyWithSymbolicTraits(
            font, 14.0, None, CoreText.kCTFontBoldTrait, CoreText.kCTFontBoldTrait
        )
        self.assertIsInstance(font2, CoreText.CTFontRef)
        self.assertResultIsCFRetained(CoreText.CTFontCreateCopyWithAttributes)

        font2 = CoreText.CTFontCreateCopyWithFamily(font, 14.0, None, "Lucida Grande")
        self.assertIsInstance(font2, CoreText.CTFontRef)
        self.assertResultIsCFRetained(CoreText.CTFontCreateCopyWithFamily)

        font2 = CoreText.CTFontCreateForString(
            font, "hello world", CoreText.CFRange(1, 4)
        )
        self.assertIsInstance(font2, CoreText.CTFontRef)
        self.assertResultIsCFRetained(CoreText.CTFontCreateForString)

        descriptor = CoreText.CTFontCopyFontDescriptor(font)
        self.assertIsInstance(descriptor, CoreText.CTFontDescriptorRef)

        v = CoreText.CTFontCopyAttribute(font, "size")
        self.assertIsNone(v)
        self.assertResultIsCFRetained(CoreText.CTFontCopyAttribute)

        v = CoreText.CTFontGetSize(font)
        self.assertIsInstance(v, float)

        v = CoreText.CTFontGetMatrix(font)
        self.assertIsInstance(v, Quartz.CGAffineTransform)

        v = CoreText.CTFontGetSymbolicTraits(font)
        self.assertIsInstance(v, int)

        v = CoreText.CTFontCopyTraits(font)
        self.assertIsInstance(v, CoreText.CFDictionaryRef)

        v = CoreText.CTFontCopyPostScriptName(font)
        self.assertIsInstance(v, str)

        v = CoreText.CTFontCopyFamilyName(font)
        self.assertIsInstance(v, str)

        v = CoreText.CTFontCopyFullName(font)
        self.assertIsInstance(v, str)

        v = CoreText.CTFontCopyDisplayName(font)
        self.assertIsInstance(v, str)

        v = CoreText.CTFontCopyName(font, CoreText.kCTFontCopyrightNameKey)
        self.assertIsInstance(v, str)

        v, locname = CoreText.CTFontCopyLocalizedName(
            font, CoreText.kCTFontCopyrightNameKey, None
        )
        self.assertIsInstance(v, (str, type(None)))
        self.assertIsInstance(locname, (str, type(None)))
        self.assertArgIsOut(CoreText.CTFontCopyLocalizedName, 2)

        v = CoreText.CTFontCopyCharacterSet(font)
        # self.assertIsInstance(v, CoreText.CFCharacterSetRef)
        self.assertIn("CharacterSet", type(v).__name__)

        v = CoreText.CTFontGetStringEncoding(font)
        self.assertIsInstance(v, int)

        v = CoreText.CTFontCopySupportedLanguages(font)
        self.assertIsInstance(v, CoreText.CFArrayRef)

        self.assertArgIsOut(CoreText.CTFontGetGlyphsForCharacters, 2)
        v, gl = CoreText.CTFontGetGlyphsForCharacters(font, "hello", None, 5)

        self.assertTrue(v is True)
        self.assertIsInstance(gl, tuple)
        self.assertEqual(len(gl), 5)
        self.assertIsInstance(gl[0], int)

        v = CoreText.CTFontGetAscent(font)
        self.assertIsInstance(v, float)

        v = CoreText.CTFontGetDescent(font)
        self.assertIsInstance(v, float)

        v = CoreText.CTFontGetLeading(font)
        self.assertIsInstance(v, float)

        v = CoreText.CTFontGetUnitsPerEm(font)
        self.assertIsInstance(v, int)

        v = CoreText.CTFontGetGlyphCount(font)
        self.assertIsInstance(v, int)

        v = CoreText.CTFontGetBoundingBox(font)
        self.assertIsInstance(v, CoreText.CGRect)

        v = CoreText.CTFontGetUnderlinePosition(font)
        self.assertIsInstance(v, float)

        v = CoreText.CTFontGetUnderlineThickness(font)
        self.assertIsInstance(v, float)

        v = CoreText.CTFontGetSlantAngle(font)
        self.assertIsInstance(v, float)

        v = CoreText.CTFontGetCapHeight(font)
        self.assertIsInstance(v, float)

        v = CoreText.CTFontGetXHeight(font)
        self.assertIsInstance(v, float)

        v = CoreText.CTFontGetGlyphWithName(font, "A")
        self.assertIsInstance(v, int)

        v, r = CoreText.CTFontGetBoundingRectsForGlyphs(
            font, CoreText.kCTFontDefaultOrientation, gl, None, 5
        )
        self.assertIsInstance(v, Quartz.CGRect)
        self.assertIsInstance(r, tuple)
        self.assertEqual(len(r), 5)
        self.assertIsInstance(r[0], Quartz.CGRect)

        v, r = CoreText.CTFontGetAdvancesForGlyphs(
            font, CoreText.kCTFontDefaultOrientation, gl, None, 5
        )
        self.assertIsInstance(v, float)
        self.assertIsInstance(r, tuple)
        self.assertEqual(len(r), 5)
        self.assertIsInstance(r[0], Quartz.CGSize)

        r = CoreText.CTFontGetVerticalTranslationsForGlyphs(font, gl, None, 5)
        self.assertIsInstance(r, tuple)
        self.assertEqual(len(r), 5)
        self.assertIsInstance(r[0], Quartz.CGSize)

        v = CoreText.CTFontCreatePathForGlyph(font, gl[0], None)
        self.assertIsInstance(v, CoreText.CGPathRef)

        self.assertResultIsCFRetained(CoreText.CTFontCopyVariationAxes)
        v = CoreText.CTFontCopyVariationAxes(font)
        if v is not None:
            self.assertIsInstance(v, CoreText.CFArrayRef)

        self.assertResultIsCFRetained(CoreText.CTFontCopyVariation)
        v = CoreText.CTFontCopyVariation(font)
        if v is not None:
            self.assertIsInstance(v, CoreText.CFDictionaryRef)

        v = CoreText.CTFontCopyFeatures(font)
        self.assertIsInstance(v, CoreText.CFArrayRef)
        self.assertResultIsCFRetained(CoreText.CTFontCopyFeatures)

        self.assertResultIsCFRetained(CoreText.CTFontCopyFeatureSettings)
        v = CoreText.CTFontCopyFeatureSettings(font)
        if v is not None:
            self.assertIsInstance(v, CoreText.CFArrayRef)

        v, o = CoreText.CTFontCopyGraphicsFont(font, None)
        self.assertIsInstance(v, CoreText.CGFontRef)
        if o is not None:
            self.assertIsInstance(o, CoreText.CTFontDescriptorRef)
        self.assertResultIsCFRetained(CoreText.CTFontCopyGraphicsFont)
        self.assertArgIsCFRetained(CoreText.CTFontCopyGraphicsFont, 1)
        self.assertArgIsOut(CoreText.CTFontCopyGraphicsFont, 1)

        v = CoreText.CTFontCreateWithGraphicsFont(v, 20.5, None, None)
        self.assertIsInstance(v, CoreText.CTFontRef)
        self.assertResultIsCFRetained(CoreText.CTFontCreateWithGraphicsFont)

        if 0:
            # Cannot tests these because the ATS framework is not
            # wrapped.
            v, o = CoreText.CTFontGetPlatformFont(font, None)
            self.assertIsInstance(v, CoreText.ATSFontRef)
            self.assertIsInstance(o, CoreText.CTFontDescriptorRef)

            v = CoreText.CTFontCreateWithPlatformFont(v, 22.5, None, None)
            self.assertIsInstance(v, CoreText.CGFontRef)

        self.assertArgIsCFRetained(CoreText.CTFontGetPlatformFont, 1)
        self.assertResultIsCFRetained(CoreText.CTFontCreateWithPlatformFont)

        v = CoreText.CTFontCopyAvailableTables(
            font, CoreText.kCTFontTableOptionNoOptions
        )
        self.assertIsInstance(v, tuple)

        v = CoreText.CTFontCopyTable(font, v[0], 0)
        self.assertIsInstance(v, CoreText.CFDataRef)

        v = CoreText.CTFontGetTypeID()
        self.assertIsInstance(v, int)

        v, a = CoreText.CTFontGetLigatureCaretPositions(font, gl[0], None, 5)
        self.assertIsInstance(v, int)
        self.assertIsInstance(a, tuple)
        self.assertEqual(len(a), 5)
        self.assertTrue(all(isinstance(x, float) for x in a))

    @min_os_level("10.6")
    def testFunctions10_6(self):
        self.assertResultIsCFRetained(CoreText.CTFontCreateWithNameAndOptions)
        v = CoreText.CTFontCreateWithNameAndOptions("Times", 15, None, 0)
        self.assertIsInstance(v, CoreText.CTFontRef)

    @min_os_level("10.6")
    def testFunctions10_6_crash(self):
        descr = CoreText.CTFontDescriptorCreateWithNameAndSize("Courier", 14.0)
        self.assertNotEqual(descr, None)

        self.assertResultIsCFRetained(CoreText.CTFontCreateWithFontDescriptorAndOptions)

        v = CoreText.CTFontCreateWithFontDescriptorAndOptions(descr, 14.0, None, 0)
        self.assertIsInstance(v, CoreText.CTFontRef)

    @min_os_level("10.7")
    def testFunctions10_7(self):
        self.assertArgHasType(CoreText.CTFontDrawGlyphs, 0, b"^{__CTFont=}")
        self.assertArgHasType(CoreText.CTFontDrawGlyphs, 1, b"n^S")
        self.assertArgSizeInArg(CoreText.CTFontDrawGlyphs, 1, 3)
        self.assertArgHasType(
            CoreText.CTFontDrawGlyphs, 2, b"n^" + CoreText.CGPoint.__typestr__
        )
        self.assertArgSizeInArg(CoreText.CTFontDrawGlyphs, 2, 3)
        self.assertArgHasType(CoreText.CTFontDrawGlyphs, 3, objc._C_NSUInteger)
        self.assertArgHasType(CoreText.CTFontDrawGlyphs, 4, b"^{CGContext=}")

    @min_os_level("10.8")
    def testFunctions10_8(self):

        font = CoreText.CTFontCreateUIFontForLanguage(
            CoreText.kCTFontMiniSystemFontType, 10.0, "nl_NL"
        )
        self.assertTrue(font is not None)
        v, gl = CoreText.CTFontGetGlyphsForCharacters(font, "hello", None, 5)
        self.assertTrue(v)
        self.assertIsInstance(gl, tuple)

        v, r = CoreText.CTFontGetOpticalBoundsForGlyphs(font, gl, None, 5, 0)
        self.assertIsInstance(v, Quartz.CGRect)
        self.assertIsInstance(r, tuple)
        self.assertEqual(len(r), 5)
        self.assertIsInstance(r[0], Quartz.CGRect)

        self.assertResultIsCFRetained(CoreText.CTFontCopyDefaultCascadeListForLanguages)

        # XXX: Introduced in the macOS 11 SDK headers
        CoreText.CTFontCopyNameForGlyph

    @min_os_level("10.9")
    def testFunctions10_9(self):
        self.assertResultIsCFRetained(CoreText.CTFontCreateForStringWithLanguage)

    @min_os_level("10.10")
    def testConstants10_10(self):
        self.assertIsInstance(CoreText.kCTFontOpenTypeFeatureTag, str)
        self.assertIsInstance(CoreText.kCTFontOpenTypeFeatureValue, str)

    @min_os_level("10.13")
    def testConstants10_13(self):
        self.assertIsInstance(CoreText.kCTFontVariationAxisHiddenKey, str)

    @min_os_level("10.15")
    def testConstants10_15(self):
        self.assertIsInstance(CoreText.kCTFontFeatureSampleTextKey, str)
        self.assertIsInstance(CoreText.kCTFontFeatureTooltipTextKey, str)



import CoreText
from PyObjCTools.TestSupport import TestCase, min_os_level
import objc


class TestCTFontDescriptor(TestCase):
    def testTypes(self):
        self.assertIsInstance(CoreText.CTFontDescriptorRef, objc.objc_class)

    def testConstants(self):
        self.assertIsInstance(CoreText.kCTFontNameAttribute, str)
        self.assertIsInstance(CoreText.kCTFontDisplayNameAttribute, str)
        self.assertIsInstance(CoreText.kCTFontFamilyNameAttribute, str)
        self.assertIsInstance(CoreText.kCTFontStyleNameAttribute, str)
        self.assertIsInstance(CoreText.kCTFontTraitsAttribute, str)
        self.assertIsInstance(CoreText.kCTFontVariationAttribute, str)
        self.assertIsInstance(CoreText.kCTFontSizeAttribute, str)
        self.assertIsInstance(CoreText.kCTFontMatrixAttribute, str)
        self.assertIsInstance(CoreText.kCTFontCascadeListAttribute, str)
        self.assertIsInstance(CoreText.kCTFontCharacterSetAttribute, str)
        self.assertIsInstance(CoreText.kCTFontLanguagesAttribute, str)
        self.assertIsInstance(CoreText.kCTFontBaselineAdjustAttribute, str)
        self.assertIsInstance(CoreText.kCTFontMacintoshEncodingsAttribute, str)
        self.assertIsInstance(CoreText.kCTFontFeaturesAttribute, str)
        self.assertIsInstance(CoreText.kCTFontFeatureSettingsAttribute, str)
        self.assertIsInstance(CoreText.kCTFontFixedAdvanceAttribute, str)
        self.assertIsInstance(CoreText.kCTFontOrientationAttribute, str)

        self.assertEqual(CoreText.kCTFontDefaultOrientation, 0)
        self.assertEqual(CoreText.kCTFontHorizontalOrientation, 1)
        self.assertEqual(CoreText.kCTFontVerticalOrientation, 2)

    @min_os_level("10.6")
    def testConstants10_6(self):
        self.assertIsInstance(CoreText.kCTFontURLAttribute, str)
        self.assertIsInstance(CoreText.kCTFontFormatAttribute, str)
        self.assertIsInstance(CoreText.kCTFontRegistrationScopeAttribute, str)
        self.assertIsInstance(CoreText.kCTFontPriorityAttribute, str)
        self.assertIsInstance(CoreText.kCTFontEnabledAttribute, str)
        self.assertIsInstance(CoreText.kCTFontEnabledAttribute, str)

        self.assertEqual(CoreText.kCTFontFormatUnrecognized, 0)
        self.assertEqual(CoreText.kCTFontFormatOpenTypePostScript, 1)
        self.assertEqual(CoreText.kCTFontFormatOpenTypeTrueType, 2)
        self.assertEqual(CoreText.kCTFontFormatTrueType, 3)
        self.assertEqual(CoreText.kCTFontFormatPostScript, 4)
        self.assertEqual(CoreText.kCTFontFormatBitmap, 5)

        self.assertEqual(CoreText.kCTFontPrioritySystem, 10000)
        self.assertEqual(CoreText.kCTFontPriorityNetwork, 20000)
        self.assertEqual(CoreText.kCTFontPriorityComputer, 30000)
        self.assertEqual(CoreText.kCTFontPriorityUser, 40000)
        self.assertEqual(CoreText.kCTFontPriorityDynamic, 50000)
        self.assertEqual(CoreText.kCTFontPriorityProcess, 60000)

    @min_os_level("10.7")
    def testConstants10_7(self):
        self.assertEqual(CoreText.kCTFontOrientationDefault, 0)
        self.assertEqual(CoreText.kCTFontOrientationHorizontal, 1)
        self.assertEqual(CoreText.kCTFontOrientationVertical, 2)
        self.assertEqual(
            CoreText.kCTFontDefaultOrientation, CoreText.kCTFontOrientationDefault
        )
        self.assertEqual(
            CoreText.kCTFontHorizontalOrientation, CoreText.kCTFontOrientationHorizontal
        )
        self.assertEqual(
            CoreText.kCTFontVerticalOrientation, CoreText.kCTFontOrientationVertical
        )

    @min_os_level("10.8")
    def testConstants10_8(self):
        self.assertEqual(CoreText.kCTFontDescriptorMatchingDidBegin, 0)
        self.assertEqual(CoreText.kCTFontDescriptorMatchingDidFinish, 1)
        self.assertEqual(CoreText.kCTFontDescriptorMatchingWillBeginQuerying, 2)
        self.assertEqual(CoreText.kCTFontDescriptorMatchingStalled, 3)
        self.assertEqual(CoreText.kCTFontDescriptorMatchingWillBeginDownloading, 4)
        self.assertEqual(CoreText.kCTFontDescriptorMatchingDownloading, 5)
        self.assertEqual(CoreText.kCTFontDescriptorMatchingDidFinishDownloading, 6)
        self.assertEqual(CoreText.kCTFontDescriptorMatchingDidMatch, 7)
        self.assertEqual(CoreText.kCTFontDescriptorMatchingDidFailWithError, 8)

        self.assertIsInstance(CoreText.kCTFontDownloadableAttribute, str)
        self.assertIsInstance(CoreText.kCTFontDescriptorMatchingSourceDescriptor, str)
        self.assertIsInstance(CoreText.kCTFontDescriptorMatchingDescriptors, str)
        self.assertIsInstance(CoreText.kCTFontDescriptorMatchingResult, str)
        self.assertIsInstance(CoreText.kCTFontDescriptorMatchingPercentage, str)
        self.assertIsInstance(CoreText.kCTFontDescriptorMatchingCurrentAssetSize, str)
        self.assertIsInstance(
            CoreText.kCTFontDescriptorMatchingTotalDownloadedSize, str
        )
        self.assertIsInstance(CoreText.kCTFontDescriptorMatchingTotalAssetSize, str)
        self.assertIsInstance(CoreText.kCTFontDescriptorMatchingError, str)

    @min_os_level("10.9")
    def testConstants10_9(self):
        self.assertIsInstance(CoreText.kCTFontOpticalSizeAttribute, str)

    @min_os_level("10.12")
    def testConstants10_12(self):
        self.assertIsInstance(CoreText.kCTFontDownloadedAttribute, str)

    @min_os_level("10.13")
    def testConstants10_13(self):
        self.assertIsInstance(CoreText.kCTFontVariationAxesAttribute, str)

    @min_os_level("10.9")
    def testFunctions10_9(self):
        CoreText.CTFontDescriptorProgressHandler = (
            objc._C_BOOL + objc._C_UINT + objc._C_ID
        )
        self.assertResultHasType(
            CoreText.CTFontDescriptorMatchFontDescriptorsWithProgressHandler,
            objc._C_BOOL,
        )
        self.assertArgIsBlock(
            CoreText.CTFontDescriptorMatchFontDescriptorsWithProgressHandler,
            2,
            CoreText.CTFontDescriptorProgressHandler,
        )

        self.assertResultIsCFRetained(CoreText.CTFontDescriptorCreateCopyWithFamily)
        self.assertResultIsCFRetained(
            CoreText.CTFontDescriptorCreateCopyWithSymbolicTraits
        )

    def testFunctions(self):

        self.assertResultIsCFRetained(CoreText.CTFontDescriptorCreateWithAttributes)
        v = CoreText.CTFontDescriptorCreateWithAttributes(
            {
                CoreText.kCTFontNameAttribute: "Optima Bold",
                CoreText.kCTFontSizeAttribute: "23.4",
            }
        )
        self.assertIsInstance(v, CoreText.CTFontDescriptorRef)

        self.assertResultIsCFRetained(CoreText.CTFontDescriptorCreateWithNameAndSize)
        v = CoreText.CTFontDescriptorCreateWithNameAndSize("Optima Bold", 15.0)
        self.assertIsInstance(v, CoreText.CTFontDescriptorRef)
        descriptor = v

        self.assertResultIsCFRetained(CoreText.CTFontDescriptorCreateCopyWithAttributes)
        v = CoreText.CTFontDescriptorCreateCopyWithAttributes(v, {"foo": "bar"})
        self.assertIsInstance(v, CoreText.CTFontDescriptorRef)

        self.assertResultIsCFRetained(CoreText.CTFontDescriptorCreateCopyWithVariation)
        v = CoreText.CTFontDescriptorCreateCopyWithVariation(v, 0, 5.0)
        self.assertIsInstance(v, CoreText.CTFontDescriptorRef)

        self.assertResultIsCFRetained(CoreText.CTFontDescriptorCreateCopyWithFeature)
        v = CoreText.CTFontDescriptorCreateCopyWithFeature(v, 0, 6)
        self.assertIsInstance(v, CoreText.CTFontDescriptorRef)

        self.assertResultIsCFRetained(
            CoreText.CTFontDescriptorCreateMatchingFontDescriptors
        )
        v = CoreText.CTFontDescriptorCreateMatchingFontDescriptors(descriptor, None)
        self.assertIsInstance(v, CoreText.CFArrayRef)

        self.assertResultIsCFRetained(
            CoreText.CTFontDescriptorCreateMatchingFontDescriptor
        )
        v = CoreText.CTFontDescriptorCreateMatchingFontDescriptor(descriptor, None)
        self.assertIsInstance(v, CoreText.CTFontDescriptorRef)

        self.assertResultIsCFRetained(CoreText.CTFontDescriptorCopyAttributes)
        v = CoreText.CTFontDescriptorCopyAttributes(descriptor)
        self.assertIsInstance(v, CoreText.CFDictionaryRef)

        self.assertResultIsCFRetained(CoreText.CTFontDescriptorCopyAttribute)
        v = CoreText.CTFontDescriptorCopyAttribute(
            descriptor, CoreText.kCTFontNameAttribute
        )
        self.assertEqual(v, "Optima Bold")

        self.assertResultIsCFRetained(CoreText.CTFontDescriptorCopyLocalizedAttribute)
        self.assertArgIsOut(CoreText.CTFontDescriptorCopyLocalizedAttribute, 2)
        v, locattr = CoreText.CTFontDescriptorCopyLocalizedAttribute(
            descriptor, CoreText.kCTFontDisplayNameAttribute, None
        )
        self.assertIsInstance(v, str)
        self.assertIsInstance(locattr, (str, type(None)))

        v = CoreText.CTFontDescriptorGetTypeID()
        self.assertIsInstance(v, int)



import CoreText
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestCTStringAttributes(TestCase):
    def testConstants(self):
        self.assertIsInstance(CoreText.kCTFontAttributeName, str)
        self.assertIsInstance(CoreText.kCTKernAttributeName, str)
        self.assertIsInstance(CoreText.kCTLigatureAttributeName, str)
        self.assertIsInstance(CoreText.kCTForegroundColorAttributeName, str)
        self.assertIsInstance(CoreText.kCTParagraphStyleAttributeName, str)
        self.assertIsInstance(CoreText.kCTUnderlineStyleAttributeName, str)
        self.assertIsInstance(CoreText.kCTVerticalFormsAttributeName, str)
        self.assertIsInstance(CoreText.kCTGlyphInfoAttributeName, str)
        self.assertEqual(CoreText.kCTUnderlineStyleNone, 0x00)
        self.assertEqual(CoreText.kCTUnderlineStyleSingle, 0x01)
        self.assertEqual(CoreText.kCTUnderlineStyleThick, 0x02)
        self.assertEqual(CoreText.kCTUnderlineStyleDouble, 0x09)
        self.assertEqual(CoreText.kCTUnderlinePatternSolid, 0x0000)
        self.assertEqual(CoreText.kCTUnderlinePatternDot, 0x0100)
        self.assertEqual(CoreText.kCTUnderlinePatternDash, 0x0200)
        self.assertEqual(CoreText.kCTUnderlinePatternDashDot, 0x0300)
        self.assertEqual(CoreText.kCTUnderlinePatternDashDotDot, 0x0400)
        self.assertEqual(CoreText.kCTWritingDirectionEmbedding, 0)
        self.assertEqual(CoreText.kCTWritingDirectionOverride, 2)

    @min_os_level("10.5")
    def testConstants10_5(self):
        self.assertIsInstance(CoreText.kCTForegroundColorFromContextAttributeName, str)
        self.assertIsInstance(CoreText.kCTSuperscriptAttributeName, str)
        self.assertIsInstance(CoreText.kCTCharacterShapeAttributeName, str)
        self.assertIsInstance(CoreText.kCTRunDelegateAttributeName, str)

    @min_os_level("10.6")
    def testConstants10_6(self):
        self.assertIsInstance(CoreText.kCTUnderlineColorAttributeName, str)
        self.assertIsInstance(CoreText.kCTStrokeWidthAttributeName, str)
        self.assertIsInstance(CoreText.kCTStrokeColorAttributeName, str)

    @min_os_level("10.8")
    def testConstants10_8(self):
        self.assertIsInstance(CoreText.kCTBaselineClassAttributeName, str)
        self.assertIsInstance(CoreText.kCTBaselineInfoAttributeName, str)
        self.assertIsInstance(CoreText.kCTBaselineReferenceInfoAttributeName, str)
        self.assertIsInstance(CoreText.kCTWritingDirectionAttributeName, str)

    @min_os_level("10.9")
    def testConstants10_9(self):
        self.assertIsInstance(CoreText.kCTLanguageAttributeName, str)

    @min_os_level("10.10")
    def testConstants10_10(self):
        self.assertIsInstance(CoreText.kCTRubyAnnotationAttributeName, str)

    @min_os_level("10.12")
    def testConstants10_12(self):
        self.assertIsInstance(CoreText.kCTBackgroundColorAttributeName, str)
        self.assertIsInstance(CoreText.kCTHorizontalInVerticalFormsAttributeName, str)
        self.assertIsInstance(CoreText.kCTTrackingAttributeName, str)

    @min_os_level("10.13")
    def testConstants10_13(self):
        self.assertIsInstance(CoreText.kCTBaselineOffsetAttributeName, str)



import CoreText
from PyObjCTools.TestSupport import TestCase, min_os_level
import objc


class TestCTFontCollection(TestCase):
    def testTypes(self):
        self.assertIsInstance(CoreText.CTFontCollectionRef, objc.objc_class)

    def testConstants(self):
        self.assertIsInstance(CoreText.kCTFontCollectionRemoveDuplicatesOption, str)

    @min_os_level("10.7")
    def testConstants10_10(self):
        self.assertIsInstance(CoreText.kCTFontCollectionIncludeDisabledFontsOption, str)
        self.assertIsInstance(
            CoreText.kCTFontCollectionDisallowAutoActivationOption, str
        )

        self.assertEqual(CoreText.kCTFontCollectionCopyDefaultOptions, 0)
        self.assertEqual(CoreText.kCTFontCollectionCopyUnique, 1 << 0)
        self.assertEqual(CoreText.kCTFontCollectionCopyStandardSort, 1 << 1)

    @min_os_level("10.7")
    def testFunctions10_7(self):
        self.assertResultIsCFRetained(CoreText.CTFontCollectionCreateMutableCopy)
        self.assertResultIsCFRetained(CoreText.CTFontCollectionCopyQueryDescriptors)
        CoreText.CTFontCollectionSetQueryDescriptors
        self.assertResultIsCFRetained(CoreText.CTFontCollectionCopyExclusionDescriptors)
        CoreText.CTFontCollectionSetExclusionDescriptors
        self.assertResultIsCFRetained(
            CoreText.CTFontCollectionCreateMatchingFontDescriptorsWithOptions
        )
        self.assertResultIsCFRetained(
            CoreText.CTFontCollectionCreateMatchingFontDescriptorsForFamily
        )
        self.assertResultIsCFRetained(CoreText.CTFontCollectionCopyFontAttribute)
        self.assertResultIsCFRetained(CoreText.CTFontCollectionCopyFontAttributes)

    def testFunctions(self):
        self.assertResultIsCFRetained(
            CoreText.CTFontCollectionCreateCopyWithFontDescriptors
        )

        v = CoreText.CTFontCollectionCreateWithFontDescriptors(
            [CoreText.CTFontDescriptorCreateWithNameAndSize("Optima Bold", 14)], None
        )
        self.assertIsInstance(v, CoreText.CTFontCollectionRef)

        col = CoreText.CTFontCollectionCreateFromAvailableFonts(None)
        self.assertIsInstance(col, CoreText.CTFontCollectionRef)

        self.assertResultIsCFRetained(
            CoreText.CTFontCollectionCreateMatchingFontDescriptors
        )
        v = CoreText.CTFontCollectionCreateMatchingFontDescriptors(col)
        self.assertIsInstance(v, CoreText.CFArrayRef)

        def compare(a, b, ctx):
            self.assertIsInstance(a, CoreText.CTFontDescriptorRef)
            self.assertIsInstance(b, CoreText.CTFontDescriptorRef)
            self.assertEqual(ctx, "foo")
            return 0

        self.assertResultIsCFRetained(
            CoreText.CTFontCollectionCreateMatchingFontDescriptorsSortedWithCallback
        )
        v = CoreText.CTFontCollectionCreateMatchingFontDescriptorsSortedWithCallback(
            col, compare, "foo"
        )
        self.assertIsInstance(v, CoreText.CFArrayRef)

        v = CoreText.CTFontCollectionGetTypeID()
        self.assertIsInstance(v, int)



"""
Python mapping for the CoreText framework.

This module does not contain docstrings for the wrapped code, check Apple's
documentation for details on how to use these functions and classes.
"""
import sys

import objc
import CoreFoundation
import Quartz
from CoreText import _metadata

sys.modules["CoreText"] = mod = objc.ObjCLazyModule(
    "CoreText",
    "com.apple.CoreText",
    objc.pathForFramework(
        "/System/Library/Frameworks/ApplicationServices.framework/Frameworks/CoreText.framework"
    ),
    _metadata.__dict__,
    None,
    {
        "__doc__": __doc__,
        "__path__": __path__,
        "__loader__": globals().get("__loader__", None),
        "objc": objc,
    },
    (CoreFoundation, Quartz),
)

import CoreText._manual as m  # isort:skip # noqa: E402

for nm in dir(m):
    setattr(mod, nm, getattr(m, nm))


del sys.modules["CoreText._metadata"]



import CoreText
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestCTTypesetter(TestCase):
    def testTypes(self):
        self.assertIsCFType(CoreText.CTTypesetterRef)

    @min_os_level("10.5")
    def testConstants10_5(self):
        self.assertIsInstance(CoreText.kCTTypesetterOptionDisableBidiProcessing, str)
        self.assertIsInstance(CoreText.kCTTypesetterOptionForcedEmbeddingLevel, str)

    @min_os_level("10.14")
    def testConstants10_14(self):
        self.assertIsInstance(CoreText.kCTTypesetterOptionAllowUnboundedLayout, str)

    def testFunctions(self):
        self.assertIsInstance(CoreText.CTTypesetterGetTypeID(), int)

        astring = CoreText.CFAttributedStringCreate(None, "hello world", None)
        self.assertIsInstance(astring, CoreText.CFAttributedStringRef)

        self.assertResultIsCFRetained(CoreText.CTTypesetterCreateWithAttributedString)
        ref = CoreText.CTTypesetterCreateWithAttributedString(astring)
        self.assertIsInstance(ref, CoreText.CTTypesetterRef)

        self.assertResultIsCFRetained(
            CoreText.CTTypesetterCreateWithAttributedStringAndOptions
        )
        options = {"key": "value"}
        ref = CoreText.CTTypesetterCreateWithAttributedStringAndOptions(
            astring, options
        )
        self.assertIsInstance(ref, CoreText.CTTypesetterRef)

        line = CoreText.CTTypesetterCreateLine(ref, CoreText.CFRange(0, 5))
        self.assertIsInstance(line, CoreText.CTLineRef)

        v = CoreText.CTTypesetterSuggestLineBreak(ref, 0, 100.0)
        self.assertIsInstance(v, int)

        v = CoreText.CTTypesetterSuggestClusterBreak(ref, 0, 100.0)
        self.assertIsInstance(v, int)

    @min_os_level("10.7")
    def testFunctions10_6(self):
        # FIXME: For some reason these aren't avaible on OSX 10.6...
        self.assertResultIsCFRetained(CoreText.CTTypesetterCreateLineWithOffset)
        CoreText.CTTypesetterSuggestLineBreakWithOffset
        CoreText.CTTypesetterSuggestLineBreak
        CoreText.CTTypesetterSuggestClusterBreakWithOffset



import CoreText
from PyObjCTools.TestSupport import TestCase, min_os_level
import objc


class TestCTFontManager(TestCase):
    @min_os_level("10.6")
    def testConstants10_6(self):
        self.assertEqual(CoreText.kCTFontManagerScopeNone, 0)
        self.assertEqual(CoreText.kCTFontManagerScopeProcess, 1)
        self.assertEqual(CoreText.kCTFontManagerScopePersistent, 2)
        self.assertEqual(CoreText.kCTFontManagerScopeSession, 3)
        self.assertEqual(
            CoreText.kCTFontManagerScopeUser, CoreText.kCTFontManagerScopePersistent
        )

        self.assertIsInstance(CoreText.kCTFontManagerBundleIdentifier, str)

        self.assertEqual(CoreText.kCTFontManagerAutoActivationDefault, 0)
        self.assertEqual(CoreText.kCTFontManagerAutoActivationDisabled, 1)
        self.assertEqual(CoreText.kCTFontManagerAutoActivationEnabled, 2)
        self.assertEqual(CoreText.kCTFontManagerAutoActivationPromptUser, 3)

        self.assertIsInstance(
            CoreText.kCTFontManagerRegisteredFontsChangedNotification, str
        )

    @min_os_level("10.6")
    def testFunctions10_6(self):
        self.assertResultIsCFRetained(
            CoreText.CTFontManagerCopyAvailablePostScriptNames
        )
        self.assertResultIsCFRetained(
            CoreText.CTFontManagerCopyAvailableFontFamilyNames
        )
        self.assertResultIsCFRetained(CoreText.CTFontManagerCopyAvailableFontURLs)
        self.assertResultIsCFRetained(
            CoreText.CTFontManagerCreateFontDescriptorsFromURL
        )

        self.assertArgHasType(
            CoreText.CTFontManagerCompareFontFamilyNames, 0, b"^{__CFString=}"
        )
        self.assertArgHasType(
            CoreText.CTFontManagerCompareFontFamilyNames, 1, b"^{__CFString=}"
        )
        self.assertArgHasType(CoreText.CTFontManagerCompareFontFamilyNames, 2, b"^v")
        self.assertArgIsOut(CoreText.CTFontManagerRegisterFontsForURL, 2)
        self.assertArgIsOut(CoreText.CTFontManagerUnregisterFontsForURL, 2)
        self.assertArgIsOut(CoreText.CTFontManagerRegisterFontsForURLs, 2)
        self.assertArgIsOut(CoreText.CTFontManagerUnregisterFontsForURLs, 2)
        self.assertArgHasType(
            CoreText.CTFontManagerEnableFontDescriptors, 1, objc._C_BOOL
        )
        CoreText.CTFontManagerGetScopeForURL  # Test that function exists
        self.assertResultHasType(CoreText.CTFontManagerIsSupportedFont, objc._C_BOOL)
        self.assertResultIsCFRetained(
            CoreText.CTFontManagerCreateFontRequestRunLoopSource
        )
        self.assertArgIsBlock(
            CoreText.CTFontManagerCreateFontRequestRunLoopSource, 1, b"@@i"
        )

        CoreText.CTFontManagerGetAutoActivationSetting
        CoreText.CTFontManagerSetAutoActivationSetting

    @min_os_level("10.7")
    def testFunctions10_7(self):
        self.assertResultIsCFRetained(
            CoreText.CTFontManagerCreateFontDescriptorFromData
        )

    @min_os_level("10.8")
    def testFunctions10_8(self):
        self.assertResultIsCFRetained(
            CoreText.CTFontManagerCreateFontDescriptorFromData
        )
        self.assertArgIsOut(CoreText.CTFontManagerRegisterGraphicsFont, 1)
        self.assertArgIsOut(CoreText.CTFontManagerUnregisterGraphicsFont, 1)

    @min_os_level("10.13")
    def testFunctions10_13(self):
        self.assertResultIsCFRetained(
            CoreText.CTFontManagerCreateFontDescriptorsFromData
        )

    @min_os_level("10.15")
    def testFunctions10_15(self):
        self.assertArgIsBlock(
            CoreText.CTFontManagerRegisterFontURLs,
            3,
            objc._C_BOOL + objc._C_ID + objc._C_BOOL,
        )
        self.assertArgIsBlock(
            CoreText.CTFontManagerUnregisterFontURLs,
            2,
            objc._C_BOOL + objc._C_ID + objc._C_BOOL,
        )
        self.assertArgIsBlock(
            CoreText.CTFontManagerRegisterFontDescriptors,
            3,
            objc._C_BOOL + objc._C_ID + objc._C_BOOL,
        )
        self.assertArgIsBlock(
            CoreText.CTFontManagerUnregisterFontDescriptors,
            2,
            objc._C_BOOL + objc._C_ID + objc._C_BOOL,
        )



import CoreText
from PyObjCTools.TestSupport import TestCase, min_os_level, cast_uint


class TestCTFontTraits(TestCase):
    def testConstants(self):
        self.assertIsInstance(CoreText.kCTFontSymbolicTrait, str)
        self.assertIsInstance(CoreText.kCTFontWeightTrait, str)
        self.assertIsInstance(CoreText.kCTFontWidthTrait, str)
        self.assertIsInstance(CoreText.kCTFontSlantTrait, str)

        self.assertEqual(CoreText.kCTFontClassMaskShift, 28)

        self.assertEqual(CoreText.kCTFontItalicTrait, cast_uint(1 << 0))
        self.assertEqual(CoreText.kCTFontBoldTrait, cast_uint(1 << 1))
        self.assertEqual(CoreText.kCTFontExpandedTrait, cast_uint(1 << 5))
        self.assertEqual(CoreText.kCTFontCondensedTrait, cast_uint(1 << 6))
        self.assertEqual(CoreText.kCTFontMonoSpaceTrait, cast_uint(1 << 10))
        self.assertEqual(CoreText.kCTFontVerticalTrait, cast_uint(1 << 11))
        self.assertEqual(CoreText.kCTFontUIOptimizedTrait, cast_uint(1 << 12))

        self.assertEqual(
            CoreText.kCTFontClassMaskTrait,
            cast_uint(15 << CoreText.kCTFontClassMaskShift),
        )

        self.assertEqual(
            CoreText.kCTFontUnknownClass, cast_uint(0 << CoreText.kCTFontClassMaskShift)
        )
        self.assertEqual(
            CoreText.kCTFontOldStyleSerifsClass,
            cast_uint(1 << CoreText.kCTFontClassMaskShift),
        )
        self.assertEqual(
            CoreText.kCTFontTransitionalSerifsClass,
            (2 << CoreText.kCTFontClassMaskShift),
        )
        self.assertEqual(
            CoreText.kCTFontModernSerifsClass,
            cast_uint(3 << CoreText.kCTFontClassMaskShift),
        )
        self.assertEqual(
            CoreText.kCTFontClarendonSerifsClass,
            cast_uint(4 << CoreText.kCTFontClassMaskShift),
        )
        self.assertEqual(
            CoreText.kCTFontSlabSerifsClass,
            cast_uint(5 << CoreText.kCTFontClassMaskShift),
        )
        self.assertEqual(
            CoreText.kCTFontFreeformSerifsClass,
            cast_uint(7 << CoreText.kCTFontClassMaskShift),
        )
        self.assertEqual(
            CoreText.kCTFontSansSerifClass,
            cast_uint(8 << CoreText.kCTFontClassMaskShift),
        )
        self.assertEqual(
            CoreText.kCTFontOrnamentalsClass,
            cast_uint(9 << CoreText.kCTFontClassMaskShift),
        )
        self.assertEqual(
            CoreText.kCTFontScriptsClass,
            cast_uint(10 << CoreText.kCTFontClassMaskShift),
        )
        self.assertEqual(
            CoreText.kCTFontSymbolicClass,
            cast_uint(12 << CoreText.kCTFontClassMaskShift),
        )

        self.assertEqual(CoreText.kCTFontTraitItalic, (1 << 0))
        self.assertEqual(CoreText.kCTFontTraitBold, (1 << 1))
        self.assertEqual(CoreText.kCTFontTraitExpanded, (1 << 5))
        self.assertEqual(CoreText.kCTFontTraitCondensed, (1 << 6))
        self.assertEqual(CoreText.kCTFontTraitMonoSpace, (1 << 10))
        self.assertEqual(CoreText.kCTFontTraitVertical, (1 << 11))
        self.assertEqual(CoreText.kCTFontTraitUIOptimized, (1 << 12))
        self.assertEqual(CoreText.kCTFontTraitColorGlyphs, (1 << 13))
        self.assertEqual(CoreText.kCTFontTraitComposite, (1 << 14))
        self.assertEqual(
            CoreText.kCTFontTraitClassMask, (15 << CoreText.kCTFontClassMaskShift)
        )

        self.assertEqual(
            CoreText.kCTFontClassUnknown, (0 << CoreText.kCTFontClassMaskShift)
        )
        self.assertEqual(
            CoreText.kCTFontClassOldStyleSerifs, (1 << CoreText.kCTFontClassMaskShift)
        )
        self.assertEqual(
            CoreText.kCTFontClassTransitionalSerifs,
            (2 << CoreText.kCTFontClassMaskShift),
        )
        self.assertEqual(
            CoreText.kCTFontClassModernSerifs, (3 << CoreText.kCTFontClassMaskShift)
        )
        self.assertEqual(
            CoreText.kCTFontClassClarendonSerifs, (4 << CoreText.kCTFontClassMaskShift)
        )
        self.assertEqual(
            CoreText.kCTFontClassSlabSerifs, (5 << CoreText.kCTFontClassMaskShift)
        )
        self.assertEqual(
            CoreText.kCTFontClassFreeformSerifs, (7 << CoreText.kCTFontClassMaskShift)
        )
        self.assertEqual(
            CoreText.kCTFontClassSansSerif, (8 << CoreText.kCTFontClassMaskShift)
        )
        self.assertEqual(
            CoreText.kCTFontClassOrnamentals, (9 << CoreText.kCTFontClassMaskShift)
        )
        self.assertEqual(
            CoreText.kCTFontClassScripts, (10 << CoreText.kCTFontClassMaskShift)
        )
        self.assertEqual(
            CoreText.kCTFontClassSymbolic, (12 << CoreText.kCTFontClassMaskShift)
        )

    @min_os_level("10.7")
    def testConstants10_7(self):
        self.assertEqual(CoreText.kCTFontTraitColorGlyphs, 1 << 13)
        self.assertEqual(CoreText.kCTFontTraitComposite, 1 << 14)

        self.assertEqual(
            CoreText.kCTFontColorGlyphsTrait, CoreText.kCTFontTraitColorGlyphs
        )

    @min_os_level("10.8")
    def testConstants10_8(self):
        self.assertEqual(CoreText.kCTFontCompositeTrait, CoreText.kCTFontTraitComposite)



import CoreText
from PyObjCTools.TestSupport import TestCase


class TestCTTextTab(TestCase):
    def testTypes(self):
        self.assertIsCFType(CoreText.CTTextTabRef)

    def testConstants(self):
        self.assertIsInstance(CoreText.kCTTabColumnTerminatorsAttributeName, str)

    def testFunctions(self):
        self.assertIsInstance(CoreText.CTTextTabGetTypeID(), int)

        tab = CoreText.CTTextTabCreate(
            CoreText.kCTCenterTextAlignment,
            10.5,
            {
                CoreText.kCTTabColumnTerminatorsAttributeName: CoreText.CFCharacterSetCreateWithCharactersInString(
                    None, "."
                )
            },
        )
        self.assertIsInstance(tab, CoreText.CTTextTabRef)

        v = CoreText.CTTextTabGetAlignment(tab)
        self.assertEqual(v, CoreText.kCTCenterTextAlignment)

        v = CoreText.CTTextTabGetLocation(tab)
        self.assertEqual(v, 10.5)

        v = CoreText.CTTextTabGetOptions(tab)
        self.assertIsInstance(v, dict)



import CoreText
from PyObjCTools.TestSupport import TestCase, min_os_level, onlyIf
import objc

try:
    from Quartz import CGSize
except ImportError:
    CGSize = None, None


class TestCTFramesetter(TestCase):
    def testTypes(self):
        self.assertIsInstance(CoreText.CTFramesetterRef, objc.objc_class)

    def testFunctions(self):
        v = CoreText.CTFramesetterGetTypeID()
        self.assertIsInstance(v, int)

        setter = CoreText.CTFramesetterCreateWithAttributedString(
            CoreText.CFAttributedStringCreate(None, "hello", None)
        )
        self.assertIsInstance(setter, CoreText.CTFramesetterRef)

        # CTFramesetterCreateFrame: tested in test_ctframe.py

        v = CoreText.CTFramesetterGetTypesetter(setter)
        self.assertIsInstance(v, CoreText.CTTypesetterRef)

    @min_os_level("10.5")
    @onlyIf(CGSize is not None, "CoreGraphics not available")
    def testMethods10_5(self):
        setter = CoreText.CTFramesetterCreateWithAttributedString(
            CoreText.CFAttributedStringCreate(None, "hello", None)
        )
        self.assertIsInstance(setter, CoreText.CTFramesetterRef)

        self.assertArgIsOut(CoreText.CTFramesetterSuggestFrameSizeWithConstraints, 4)

        r = CoreText.CTFramesetterSuggestFrameSizeWithConstraints(
            setter, CoreText.CFRange(0, 2), None, CGSize(100, 500), None
        )
        self.assertIsInstance(r, tuple)
        self.assertEqual(len(r), 2)

        size, a_range = r

        self.assertIsInstance(size, CGSize)
        self.assertIsInstance(a_range, CoreText.CFRange)

    @min_os_level("10.14")
    def testMethods10_14(self):
        self.assertResultIsCFRetained(CoreText.CTFramesetterCreateWithTypesetter)



import os

import CoreText
import Quartz
from PyObjCTools.TestSupport import TestCase, min_os_level
import objc


class TestCTFrame(TestCase):
    def testTypes(self):
        self.assertIsInstance(CoreText.CTFrameRef, objc.objc_class)

    def testConstants(self):
        self.assertEqual(CoreText.kCTFrameProgressionTopToBottom, 0)
        self.assertEqual(CoreText.kCTFrameProgressionRightToLeft, 1)
        self.assertEqual(CoreText.kCTFrameProgressionLeftToRight, 2)

        self.assertEqual(CoreText.kCTFramePathFillEvenOdd, 0)
        self.assertEqual(CoreText.kCTFramePathFillWindingNumber, 1)

        self.assertIsInstance(CoreText.kCTFrameProgressionAttributeName, str)

    @min_os_level("10.7")
    def testConstants10_7(self):
        self.assertIsInstance(CoreText.kCTFramePathFillRuleAttributeName, str)
        self.assertIsInstance(CoreText.kCTFramePathWidthAttributeName, str)
        self.assertIsInstance(CoreText.kCTFrameClippingPathsAttributeName, str)
        self.assertIsInstance(CoreText.kCTFramePathClippingPathAttributeName, str)

    def testFunctions(self):
        v = CoreText.CTFrameGetTypeID()
        self.assertIsInstance(v, int)

        setter = CoreText.CTFramesetterCreateWithAttributedString(
            CoreText.CFAttributedStringCreate(None, "hello", None)
        )
        self.assertIsInstance(setter, CoreText.CTFramesetterRef)

        path = CoreText.CGPathCreateMutable()
        self.assertIsInstance(path, CoreText.CGPathRef)
        CoreText.CGPathAddRect(
            path,
            None,
            CoreText.CGRect(CoreText.CGPoint(0, 0), CoreText.CGSize(400, 400)),
        )
        attr = {"foo": 42}
        frame = CoreText.CTFramesetterCreateFrame(
            setter, CoreText.CFRange(0, 5), path, attr
        )
        self.assertIsInstance(frame, CoreText.CTFrameRef)

        v = CoreText.CTFrameGetStringRange(frame)
        self.assertIsInstance(v, CoreText.CFRange)

        v = CoreText.CTFrameGetVisibleStringRange(frame)
        self.assertIsInstance(v, CoreText.CFRange)

        self.assertResultIsNotCFRetained(CoreText.CTFrameGetPath)
        v = CoreText.CTFrameGetPath(frame)
        self.assertIsInstance(v, CoreText.CGPathRef)

        v = CoreText.CTFrameGetFrameAttributes(frame)
        self.assertTrue(v is attr)

        v = CoreText.CTFrameGetLines(frame)
        self.assertIsInstance(v, CoreText.CFArrayRef)

        self.assertArgIsOut(CoreText.CTFrameGetLineOrigins, 2)
        self.assertArgSizeInArg(CoreText.CTFrameGetLineOrigins, 2, 1)
        v = CoreText.CTFrameGetLineOrigins(frame, CoreText.CFRange(0, 1), None)
        self.assertIsInstance(v, tuple)
        self.assertEqual(len(v), 1)
        self.assertIsInstance(v[0], CoreText.CGPoint)

        url = CoreText.CFURLCreateWithFileSystemPath(
            None, "/tmp/pyobjc.test.pdf", CoreText.kCFURLPOSIXPathStyle, False
        )
        self.assertIsInstance(url, CoreText.CFURLRef)
        ctx = Quartz.CGPDFContextCreateWithURL(
            url, Quartz.CGRect(CoreText.CGPoint(0, 0), Quartz.CGSize(1000, 1000)), None
        )
        self.assertIsInstance(ctx, Quartz.CGContextRef)

        CoreText.CTFrameDraw(frame, ctx)

        if os.path.exists("/tmp/pyobjc.test.pdf"):
            os.unlink("/tmp/pyobjc.test.pdf")



import CoreText
from PyObjCTools.TestSupport import TestCase, min_os_level, expectedFailure
import objc


class TestCTRunDelegate(TestCase):
    @min_os_level("10.9")
    def testTypes(self):
        self.assertIsCFType(CoreText.CTRunDelegateRef)

    @min_os_level("10.9")
    def testConstants(self):
        self.assertEqual(CoreText.kCTRunDelegateVersion1, 1)
        self.assertEqual(
            CoreText.kCTRunDelegateCurrentVersion, CoreText.kCTRunDelegateVersion1
        )

    @expectedFailure
    @min_os_level("10.9")
    def testFunctions(self):
        self.assertIsInstance(CoreText.CTRunDelegateGetTypeID(), int)

        self.assertNotIsInstance(CoreText.CTRunDelegateCreate, objc.function)
        self.assertNotIsInstance(CoreText.CTRunDelegateGetRefCon, objc.function)

        def getAscender(info):
            return info["ascender"]

        def getDescender(info):
            return info["descender"]

        def getWidth(info):
            return info["width"]

        rc = {"ascender": 1.0, "descender": 2.0, "width": 3.0}

        o = CoreText.CTRunDelegateCreate((getAscender, getDescender, getWidth), rc)
        self.assertIsInstance(o, CoreText.CTRunDelegateRef)

        # v = CoreText.CTRunDelegateGetRefCon(o)
        # self.assertIs(v, rc)

        # XXX: Still missing tests that actually call the callback
        #      functions.
        self.fail("Tests for using callbacks!")



import CoreText
from PyObjCTools.TestSupport import TestCase, min_os_level
import objc


class TestCTGlyphInfo(TestCase):
    def testTypes(self):
        self.assertIsInstance(CoreText.CTGlyphInfoRef, objc.objc_class)

    def testConstants(self):
        self.assertEqual(CoreText.kCTIdentityMappingCharacterCollection, 0)
        self.assertEqual(CoreText.kCTAdobeCNS1CharacterCollection, 1)
        self.assertEqual(CoreText.kCTAdobeGB1CharacterCollection, 2)
        self.assertEqual(CoreText.kCTAdobeJapan1CharacterCollection, 3)
        self.assertEqual(CoreText.kCTAdobeJapan2CharacterCollection, 4)
        self.assertEqual(CoreText.kCTAdobeKorea1CharacterCollection, 5)

        self.assertEqual(CoreText.kCTCharacterCollectionIdentityMapping, 0)
        self.assertEqual(CoreText.kCTCharacterCollectionAdobeCNS1, 1)
        self.assertEqual(CoreText.kCTCharacterCollectionAdobeGB1, 2)
        self.assertEqual(CoreText.kCTCharacterCollectionAdobeJapan1, 3)
        self.assertEqual(CoreText.kCTCharacterCollectionAdobeJapan2, 4)
        self.assertEqual(CoreText.kCTCharacterCollectionAdobeKorea1, 5)

    def testFunctions(self):
        v = CoreText.CTGlyphInfoGetTypeID()
        self.assertIsInstance(v, int)

        font = CoreText.CTFontCreateWithName("Optima Bold", 14, None)
        self.assertIsInstance(font, CoreText.CTFontRef)

        self.assertResultIsCFRetained(CoreText.CTGlyphInfoCreateWithGlyphName)
        info = v = CoreText.CTGlyphInfoCreateWithGlyphName("copyright", font, "(c)")
        self.assertIsInstance(v, CoreText.CTGlyphInfoRef)

        self.assertResultIsCFRetained(CoreText.CTGlyphInfoCreateWithGlyph)
        # v = CTGlyphInfoCreateWithGlyph(3254, font, "(c)")
        # self.assertIsInstance(v, CTGlyphInfoRef)

        self.assertResultIsCFRetained(CoreText.CTGlyphInfoCreateWithCharacterIdentifier)

        for collection in (
            CoreText.kCTIdentityMappingCharacterCollection,
            CoreText.kCTAdobeCNS1CharacterCollection,
            CoreText.kCTAdobeGB1CharacterCollection,
            CoreText.kCTAdobeJapan1CharacterCollection,
            CoreText.kCTAdobeJapan2CharacterCollection,
            CoreText.kCTAdobeKorea1CharacterCollection,
        ):
            v = CoreText.CTGlyphInfoCreateWithCharacterIdentifier(
                3254, collection, "(c)"
            )
            if v is not None:
                break
        self.assertIsInstance(v, CoreText.CTGlyphInfoRef)

        v = CoreText.CTGlyphInfoGetGlyphName(info)
        self.assertIsInstance(v, str)

        v = CoreText.CTGlyphInfoGetCharacterIdentifier(info)
        self.assertIsInstance(v, int)

        self.assertResultIsNotCFRetained(CoreText.CTGlyphInfoGetCharacterCollection)
        v = CoreText.CTGlyphInfoGetCharacterCollection(info)
        self.assertIsInstance(v, int)

    @min_os_level("10.15")
    def testFunctions10_15(self):
        CoreText.CTGlyphInfoGetGlyph
